# Copyright (c) 2012-2013 by Enthought Inc.

import contextlib
import sys
from unittest.case import _ExpectedFailure, _UnexpectedSuccess

from traits.api import (
    Any, Event, HasStrictTraits, Instance, Int, List, Property, Str,
)
from traits.testing.unittest_tools import UnittestTools, reverse_assertion



@contextlib.contextmanager
def expected_failure():
    """ An expected failure context manager. The executed block will only be
    considered an expected failure if there is an assertion raised. Else if
    an exception is raised the error is re-raised. Finally if there was no
    exception the block is marked as an unexpected success.

    """
    try:
        yield
    except AssertionError:
        raise _ExpectedFailure(sys.exc_info())
    except Exception:
        raise
    else:
        raise _UnexpectedSuccess

class _AssertAtomChangesContext(object):
    """ Context manager used to implement TestAssistant.assertAtomChanges.

    """
    def __init__(self, obj, xname, count, test_case):
        self.obj = obj
        self.xname = xname
        self.count = count
        self.event = None
        self.events = []
        self.failureException = test_case.failureException

    def _listener(self, change):
        """ Dummy atom listener.

        """
        obj = change['object']
        name = change['name']
        value = change['value']
        self.event = (obj, name, value)
        self.events.append(self.event)

    def __enter__(self):
        """ Bind the trait listener.

        """
        self.obj.observe(self.xname, self._listener)
        return self

    def __exit__(self, exc_type, exc_value, tb):
        """ Remove the trait listener.

        """
        if exc_type is not None:
            return False

        self.obj.unobserve(self.xname)
        if self.event is None:
            msg = 'A change event was not fired for: {0}'.format(self.xname)
            raise self.failureException(msg)
        elif self.count is not None and len(self.events) != self.count:
            msg = 'Change event for {0} was fired {1} times instead of {2}'
            items = self.xname, len(self.events), self.count
            raise self.failureException(msg.format(*items))

        return False

class TestAssistant(UnittestTools):
    """Mixin class to augment the unittest.TestCase with useful traits and atom
    assert methods.

    """

    ### Trait assertion methods ########################################

    def assertAtomChanges(self, obj, trait, count=None, callableObj=None,
                           *args, **kwargs):
        """ Same method as assertTraitChanges but for Atom based object ...

        """

        context = _AssertAtomChangesContext(obj, trait, count, self)
        if callableObj is None:
            return context
        with context:
            callableObj(*args, **kwargs)

    def assertAtomDoesNotChange(self, obj, atom, callableObj=None,
                                 *args, **kwargs):
        """Assert an object atom does not change.

        Assert that the class atom does not change during execution of the
        provided function.

        """
        msg = 'A change event was fired for: {0}'.format(atom)
        context = _AssertAtomChangesContext(obj, atom, None, self)
        if callableObj is None:
            return reverse_assertion(context, msg)
        with reverse_assertion(context, msg):
            callableObj(*args, **kwargs)




#
# (C) Copyright 2013 Enthought, Inc., Austin, TX
# All right reserved.
#
# This file is open source software distributed according to the terms in
# LICENSE.txt
#
from traits.etsconfig.api import ETSConfig

import enaml

from .trait_operators import TRAIT_OPERATORS

if ETSConfig.toolkit not in ['', 'qt4']:
    raise ValueError('traits-enaml does not support WX')

ETSConfig.toolkit = 'qt4'

def imports():
    return enaml.imports(operators=TRAIT_OPERATORS)



#------------------------------------------------------------------------------
#  Copyright (c) 2013, Enthought, Inc.
#  All rights reserved.
#------------------------------------------------------------------------------
from atom.api import Typed, observe, set_default

from enaml.widgets.api import RawWidget
from enaml.core.declarative import d_

from traits.api import HasTraits, Instance
from traitsui.api import View, Item

from tvtk.pyface.scene_editor import SceneEditor
from tvtk.pyface.scene_model import SceneModel


class MayaviModel(HasTraits):
    scene = Instance(SceneModel, ())
    view = View(Item('scene', editor=SceneEditor(), resizable=True,
                     show_label=False),
                resizable=True)


class MayaviCanvas(RawWidget):
    """ A widget that displays a mayavi scene.

    :Attributes:
        **scene** = *d_(Typed(SceneModel))*
            The mayavi scene model to be displayed.

    """
    #: The mayavi scene model to be displayed.
    scene = d_(Typed(SceneModel))

    #: The traits model that is used to create the mayavi UI
    _model = MayaviModel()

    #: Mayavi canvas' expand freely in width and height by default
    hug_width = set_default('ignore')
    hug_height = set_default('ignore')

    def initialize(self):
        super(MayaviCanvas, self).initialize()
        if self.scene is not None:
            self._model.scene = self.scene

    def create_widget(self, parent):
        ui = self._model.edit_traits(kind='subpanel')
        widget = ui.control
        widget.setParent(parent)

        return widget

    @observe('scene')
    def scene_changed(self, change):
        self._model.scene = change['value']



#
# (C) Copyright 2013 Enthought, Inc., Austin, TX
# All right reserved.
#
# This file is open source software distributed according to the terms in
# LICENSE.txt
#

from collections import namedtuple
from datetime import date, time

from traits.api import (BaseInstance, Bool, Button, Enum, Event, Float, Int,
                        Range, Str)
import traits_enaml

with traits_enaml.imports():
    from enaml.widgets.api import Label
    from traits_enaml.widgets.auto_editors import (AutoView, AutoWindow,
                                                   BoolEditor, ButtonEditor,
                                                   DateEditor, EnumEditor,
                                                   FloatEditor,
                                                   FloatRangeEditor, IntEditor,
                                                   IntRangeEditor, StrEditor,
                                                   TimeEditor, DefaultEditor)

TraitDesc = namedtuple('TraitDesc', 'name trait_type label tooltip editor')


def auto_view(model):
    """ Generate a view directly from a `HasTraits` instance.
    """
    descriptions = _model_traits(model)
    labels = [Label(text=desc.label) for desc in descriptions]
    editors = [_get_editor(model, desc) for desc in descriptions]
    objects = [widget for pair in zip(labels, editors) for widget in pair]
    return AutoView(objects=objects)


def auto_window(model):
    """ Generate a window directly from a `HasTraits` instance.
    """
    return AutoWindow(view=auto_view(model))


def _get_editor(model, trait_desc):
    kwargs = {'model': model, 'trait_desc': trait_desc}
    if trait_desc.tooltip:
        kwargs['tool_tip'] = trait_desc.tooltip

    if trait_desc.editor:
        return trait_desc.editor(**kwargs)
    elif isinstance(trait_desc.trait_type, Bool):
        return BoolEditor(**kwargs)
    elif isinstance(trait_desc.trait_type, Button):
        return ButtonEditor(**kwargs)
    elif (isinstance(trait_desc.trait_type, BaseInstance) and
            trait_desc.trait_type.klass is date):
        return DateEditor(**kwargs)
    elif isinstance(trait_desc.trait_type, Enum):
        return EnumEditor(**kwargs)
    elif isinstance(trait_desc.trait_type, Float):
        return FloatEditor(**kwargs)
    elif (isinstance(trait_desc.trait_type, Range) and
            (isinstance(trait_desc.trait_type._low, float) or
             isinstance(trait_desc.trait_type._high, float))):
        return FloatRangeEditor(**kwargs)
    elif isinstance(trait_desc.trait_type, Int):
        return IntEditor(**kwargs)
    elif (isinstance(trait_desc.trait_type, Range) and
            isinstance(trait_desc.trait_type._low, int) and
            isinstance(trait_desc.trait_type._high, int)):
        return IntRangeEditor(**kwargs)
    elif isinstance(trait_desc.trait_type, Str):
        return StrEditor(**kwargs)
    elif (isinstance(trait_desc.trait_type, BaseInstance) and
            trait_desc.trait_type.klass is time):
        return TimeEditor(**kwargs)

    return DefaultEditor(**kwargs)


def _model_traits(model):
    traits = []
    for name in model.class_trait_names():
        trait = model.trait(name)
        if type(trait.trait_type) is Event:
            continue
        label = trait.label or " ".join(name.split('_')).capitalize()
        tooltip = trait.tooltip
        editor = trait.enaml_editor
        desc = TraitDesc(name, trait.trait_type, label, tooltip, editor)
        traits.append(desc)
    return traits



import unittest

from traits.api import Event, HasTraits, Str
from traits_enaml.testing.enaml_test_assistant import EnamlTestAssistant

class TraitModel(HasTraits):
    value_delegate = Str()
    value_subscribe = Str()
    value_update = Str()
    value_simple = Str('simple_text')
    value_notify = Event()

class TraitOperatorsTestCase(EnamlTestAssistant, unittest.TestCase):

    def setUp(self):

        EnamlTestAssistant.setUp(self)

        enaml_source = """
from enaml.widgets.api import MainWindow, Field

enamldef MainView(MainWindow):
    attr model
    Field:
        name = 'test_op_delegate'
        text := model.value_delegate
    Field:
        name = 'test_op_subscribe'
        text << model.value_subscribe
    Field:
        name = 'test_op_update'
        text >> model.value_update
    Field:
        name = 'test_op_simple'
        text = model.value_simple
    Field:
        name = 'test_op_notify'
        text ::
            model.value_notify = True
"""
        self.model = TraitModel()
        view, toolkit_view = self.parse_and_create(
            enaml_source, model=self.model
        )

        self.view = view

    def tearDown(self):
        self.view = None
        self.model = None
        EnamlTestAssistant.tearDown(self)


    def test_op_delegate(self):

        enaml_widget = self.view.find('test_op_delegate')

        with self.assertTraitChanges(self.model, 'value_delegate'):
            enaml_widget.text = 'new_value'

        self.assertEquals(self.model.value_delegate, 'new_value')

        with self.assertAtomChanges(enaml_widget, 'text'):
            self.model.value_delegate = 'updated_trait'

        self.assertEquals(enaml_widget.text, 'updated_trait')

    def test_op_subscribe(self):

        enaml_widget = self.view.find('test_op_subscribe')

        with self.assertTraitDoesNotChange(self.model, 'value_subscribe'):
            enaml_widget.text = 'new_value'

        with self.assertAtomChanges(enaml_widget, 'text'):
            self.model.value_subscribe = 'updated_trait'

        self.assertEquals(enaml_widget.text, 'updated_trait')

    def test_op_update(self):

        enaml_widget = self.view.find('test_op_update')

        with self.assertTraitChanges(self.model, 'value_update'):
            enaml_widget.text = 'new_value'

        self.assertEquals(self.model.value_update, 'new_value')

        with self.assertAtomDoesNotChange(enaml_widget, 'text'):
            self.model.value_subscribe = 'updated_trait'

    def test_op_simple(self):

        enaml_widget = self.view.find('test_op_simple')

        self.assertEquals(self.model.value_simple, enaml_widget.text)
        self.assertEquals('simple_text', enaml_widget.text)

        with self.assertTraitDoesNotChange(self.model, 'value_simple'):
            enaml_widget.text = 'new_value'

        with self.assertAtomDoesNotChange(enaml_widget, 'text'):
            self.model.value_simple = 'updated_trait'

    def test_op_notify(self):

        enaml_widget = self.view.find('test_op_notify')

        with self.assertTraitChanges(self.model, 'value_notify'):
            enaml_widget.text = 'changing text'

        # Updating the text with the same value does not trigger the event
        with self.assertTraitDoesNotChange(self.model, 'value_notify'):
            enaml_widget.text = 'changing text'

        with self.assertTraitChanges(self.model, 'value_notify'):
            enaml_widget.text = 'new text'




# Copyright (c) 2011-2013 by Enthought, Inc.
# All rights reserved.

from setuptools import setup, find_packages


setup(
    name='traits-enaml',
    version='0.2.0',
    author='Enthought, Inc',
    author_email='info@enthought.com',
    url='https://github.com/enthought/traits-enaml',
    description='Utilities for interoperation between Traits and Enaml',
    long_description=open('README.md').read(),
    packages=find_packages(),
    package_data={'traits_enaml.testing.tests': ['*.enaml'],
                  'traits_enaml.widgets': ['*.enaml']},
    requires=[
    ],
)
